import { MetadataMap, KeyValueMap } from "./node.js";
export declare enum Errors {
    Undetermined = -1,
    Success = 0,
    NodeNotFound = 1,
    SecondNodeNotFound = 2,
    PropertyNotFound = 3,
    PortKeyNotFound = 4,
    SecondPortKeyNotFound = 5,
    InvalidComponentKey = 6,
    ModuleNotFound = 7,
    InvalidConnectionId = 8,
    GroupNotFound = 9
}
export declare enum Actions {
    CreateNode = "create-node",
    RemoveNode = "remove-node",
    CreateConnection = "create-connection",
    RemoveConnection = "remove-connection",
    SetProperty = "set-property",
    RemoveProperty = "remove-property",
    SetMetadata = "set-metadata",
    RemoveMetadata = "remove-metadata",
    MoveNodesToModule = "move-nodes-to-module",
    SwapConnections = "swap-connections",
    CreateGroup = "create-group",
    RemoveGroup = "remove-group",
    AddNodesToGroup = "add-nodes-to-group",
    RemoveNodesFromGroup = "remove-nodes-from-group",
    CopyNodes = "copy-nodes",
    PasteNodes = "paste-nodes"
}
export declare const ORIGIN_PATCH_KEY = "_origin_patch_";
export declare const ORIGIN_PROJECT_KEY = "_origin_project_";
export declare type CreateNodeActionData = {
    componentKey: string;
    properties?: KeyValueMap;
    metadata?: MetadataMap;
};
export declare type RemoveNodeActionData = {
    nodeId: number;
};
export declare type CreateConnectionActionData = {
    fromNodeId: number;
    fromPortKey: string;
    toNodeId: number;
    toPortKey: string;
    toIndex?: number;
};
export declare type RemoveConnectionActionData = {
    fromNodeId: number;
    fromPortKey: string;
    connectionId: number;
};
export declare type SetPropertyActionData = {
    nodeId: number;
    propertyKey: string;
    value: any;
};
export declare type RemovePropertyActionData = {
    nodeId: number;
    propertyKey: string;
};
export declare type SetMetadataActionData = {
    nodeId: number;
    key: string;
    value: any;
};
export declare type RemoveMetadataActionData = {
    nodeId: number;
    key: string;
};
export declare type MoveNodesToModuleActionData = {
    moduleId: number;
    nodeIdList: number[];
};
export declare type SwapConnectionsActionData = {
    nodeId: number;
    portKey: string;
    sourceConnectionId: number;
    targetConnectionId: number;
};
export declare type CreateGroupActionData = {
    name: string;
    nodeIdList?: number[];
};
export declare type RemoveGroupActionData = {
    groupId: number;
};
export declare type AddNodesToGroupActionData = {
    groupId: number;
    nodeIdList: number[];
};
export declare type RemoveNodesFromGroupActionData = {
    groupId: number;
    nodeIdList: number[];
};
export declare type CopyNodesActionData = {
    nodeIdList: number[];
};
export declare type PasteNodesActionData = {};
export declare const hasCopiedNodes: () => boolean;
