import { getComponent } from "../component_manager.js";
import { NodeType } from "../node.js";
import { setProperty } from "../utils.js";
const INTERNAL_SIGNALGATE_COMPONENT_KEY = "internal.signal.gate";
const INTERNAL_SIGNALGATE_PORT_KEY = "signal";
class Sequencer {
    constructor(project, patch) {
        this.project = project;
        this.pathTrackingMap = {};
        this.patch = patch;
    }
    getInputCount(node) {
        let count = 0;
        for (let connections of Object.values(node.inputs || [])) {
            count += connections.length;
        }
        return count;
    }
    getStartNodes(nodes) {
        let startNodes = [];
        for (let node of Object.values(nodes)) {
            let component = getComponent(node.componentKey);
            if (node.type == NodeType.ModuleInput || node.type == NodeType.ModuleOutput) {
                continue;
            }
            // Collect start nodes in modules
            else if (node.type == NodeType.Module) {
                startNodes = [...startNodes, ...this.getStartNodes(node.patch.nodes)];
                continue;
            }
            else {
                if ((component === null || component === void 0 ? void 0 : component.data.inputs) != null) {
                    continue;
                }
            }
            startNodes.push(node);
        }
        return startNodes;
    }
    track() {
        var _a;
        let __race_count = 0;
        let patchNodes = this.patch.nodes;
        let nodes = this.getStartNodes(this.patch.nodes);
        let depth = 0;
        let result = {
            startNodes: [...nodes],
            mainSequence: [],
            signalSequences: {},
        };
        while (nodes.length > 0) {
            if (__race_count > 999999) {
                throw ("** Infinite loop detected in sequencer track function.");
            }
            let node = nodes.shift();
            let component = getComponent(node.componentKey);
            let currentSignalSource = node.id in this.pathTrackingMap ? this.pathTrackingMap[node.id].signalSource : null;
            for (let [portKey, connections] of Object.entries(node.outputs || [])) {
                let signalSource = ((_a = component === null || component === void 0 ? void 0 : component.getOutputPort(portKey)) === null || _a === void 0 ? void 0 : _a.signal)
                    ? {
                        node: node,
                        portKey: portKey,
                    }
                    : currentSignalSource;
                for (let connection of connections) {
                    let targetNode = patchNodes[connection.toNodeId];
                    if (!(targetNode.id in this.pathTrackingMap)) {
                        // Init tracking node entry
                        this.pathTrackingMap[targetNode.id] = {
                            counter: this.getInputCount(targetNode),
                            depth: -1,
                            signalSource: signalSource
                        };
                    }
                    else {
                        if (signalSource) {
                            if (this.pathTrackingMap[targetNode.id].signalSource) {
                                if (signalSource.node.id != this.pathTrackingMap[targetNode.id].signalSource.node.id
                                    || signalSource.portKey != this.pathTrackingMap[targetNode.id].signalSource.portKey) {
                                    // Merge signals, insert internal signal gate component
                                    let gateNode = {
                                        "componentKey": INTERNAL_SIGNALGATE_COMPONENT_KEY,
                                        "name": "",
                                        "id": this.project.idGenerator.next(),
                                        "type": NodeType.Internal,
                                    };
                                    setProperty(result.signalSequences, [targetNode.id, portKey, []], gateNode);
                                    //result.signalSequences[targetNode.id][portKey].push(gateNode);
                                    setProperty(result.signalSequences, [signalSource.node.id, signalSource.portKey, []], gateNode);
                                    //result.signalSequences[signalSource.node.id][signalSource.portKey].push(gateNode);
                                    signalSource = this.pathTrackingMap[targetNode.id].signalSource = {
                                        node: gateNode,
                                        portKey: INTERNAL_SIGNALGATE_PORT_KEY
                                    };
                                }
                            }
                            else {
                                this.pathTrackingMap[targetNode.id].signalSource = signalSource;
                            }
                        }
                    }
                    //
                    if (this.pathTrackingMap[targetNode.id].counter > 0) {
                        this.pathTrackingMap[targetNode.id].counter--;
                    }
                    //
                    if (this.pathTrackingMap[targetNode.id].counter == 0) {
                        nodes.push(targetNode);
                        if (signalSource == null) {
                            result.mainSequence.push(targetNode);
                        }
                        else {
                            setProperty(result.signalSequences, [signalSource.node.id, signalSource.portKey, []], targetNode);
                        }
                    }
                }
            }
        }
        return result;
    }
}
export default Sequencer;
//# sourceMappingURL=sequencer.js.map