import Component from "./component.js";
import Node from "./node.js";
export interface Port {
    name: string;
    type: string;
}
export interface Input extends Port {
    isMulti?: boolean;
    required?: boolean;
}
export interface InputMap {
    [portKey: string]: Input;
}
export interface Output extends Port {
    signal?: boolean;
}
export interface OutputMap {
    [portKey: string]: Output;
}
export interface Property {
    name: string;
    type: string;
    defaultValue: any;
}
export interface Properties {
    [key: string]: Property;
}
export declare enum Appearance {
    Normal = 0,
    Simple = 1
}
declare type EvaluatorCallback = (node: Node) => string;
export interface ComponentData {
    name: string;
    icon?: string;
    appearance?: Appearance;
    description?: string;
    inputs?: InputMap;
    outputs?: OutputMap;
    properties?: Properties;
    evaluator?: string | EvaluatorCallback;
    signalEvaluator?: string | EvaluatorCallback;
    onEvaluate?: {
        [name: string]: (node: Node) => string | undefined;
    };
}
interface ComponentsMap {
    [key: string]: Component;
}
export interface NamespaceData {
    name: string;
    defaultIcon?: string;
    description: string;
}
interface NamespaceMap {
    [key: string]: Namespace;
}
export interface NamespaceTree {
    [key: string]: {
        branch: NamespaceTree;
        namespace: Namespace;
    };
}
export declare let components: ComponentsMap;
export declare let namespaces: NamespaceMap;
export declare const splitIdentKey: (key: string) => string[];
export declare const getIdentKey: (nsKey: string, componentKey: string) => string;
export declare class Namespace {
    key: string;
    components: ComponentsMap;
    data: NamespaceData;
    constructor(key: string, data: NamespaceData);
    setRequiredAsDefault(componentData: ComponentData): void;
    registerComponent(key: string, componentData: ComponentData): void;
    getComponent(key: string): Component;
}
export declare const getComponent: (key: string) => Component;
export declare const registerNamespace: (key: string, data: NamespaceData) => Namespace;
export declare const getNamespaceTree: () => NamespaceTree;
export {};
