import Component from "./component.js";
export var Appearance;
(function (Appearance) {
    Appearance[Appearance["Normal"] = 0] = "Normal";
    Appearance[Appearance["Simple"] = 1] = "Simple";
})(Appearance || (Appearance = {}));
export let components = {};
export let namespaces = {};
// export const getPortType = (dir: "input" | "output", componentData: ComponentData, port: string) => {
//     return dir == "input" ?
//         port == flowInput ? flowInput : componentData.inputs?.[port]?.type :
//         port == flowOutput ? flowOutput : componentData.outputs?.[port]?.type;
// }
export const splitIdentKey = (key) => {
    let index = key.lastIndexOf(".");
    let nsKey = key.slice(0, index);
    let comKey = key.slice(index + 1);
    return [nsKey, comKey];
};
// export const getComponentFullName = (key: string) => {
//     let [nsKey, comKey] = splitIdentKey(key);
//     let namespace = namespaces[nsKey];
//     return namespace.data.name + "." + namespace.getComponent(comKey).data.name;
// }
export const getIdentKey = (nsKey, componentKey) => {
    return nsKey + "." + componentKey;
};
export class Namespace {
    constructor(key, data) {
        this.components = {};
        this.key = key;
        this.data = data;
    }
    setRequiredAsDefault(componentData) {
        if (componentData.inputs) {
            for (let input of Object.values(componentData.inputs)) {
                if (input.required == null) {
                    input.required = true;
                }
            }
        }
    }
    registerComponent(key, componentData) {
        this.setRequiredAsDefault(componentData);
        this.components[key] = new Component(this, key, componentData);
    }
    getComponent(key) {
        let data = this.components[key];
        return data;
    }
}
export const getComponent = (key) => {
    let [nsKey, comKey] = splitIdentKey(key);
    let namespace = namespaces[nsKey];
    return namespace.getComponent(comKey);
};
export const registerNamespace = (key, data) => {
    let namespace = new Namespace(key, data);
    namespaces[key] = namespace;
    return namespace;
};
export const getNamespaceTree = () => {
    let tree = {};
    let keys = Object.keys(namespaces).sort();
    for (let key of keys) {
        let namespace = namespaces[key];
        let keyPath = key.split(".");
        let t = tree;
        for (let key of keyPath) {
            if (!(key in t)) {
                t[key] = {
                    branch: {},
                    namespace: namespace
                };
            }
            t = t[key].branch;
        }
    }
    return tree;
};
//# sourceMappingURL=component_manager.js.map