import { registerType } from "../type_manager.js";
import { Appearance, registerNamespace } from "./../component_manager.js";
registerType("number", {
    name: "Number",
    defaultComponent: "core.math.constant",
    validator: (x) => { return isNaN(+x); }
});
let namespace = registerNamespace("core.math", {
    name: "Math",
    defaultIcon: "calculate",
    description: ""
});
namespace.registerComponent("constant", {
    name: "Constanta",
    description: "",
    appearance: Appearance.Simple,
    outputs: {
        "value": {
            name: "Value",
            type: "number",
        }
    },
    properties: {
        value: {
            name: "Value",
            type: "number",
            defaultValue: 0
        }
    },
    evaluator: "@{out:value}=@{prop:value}"
});
let arithmeticBase = {
    inputs: {
        a: {
            name: "A",
            type: "number",
            required: true,
        },
        b: {
            name: "B",
            type: "number",
            required: true,
        }
    },
    outputs: {
        result: {
            name: "Result",
            type: "number"
        }
    },
};
namespace.registerComponent("add", {
    ...arithmeticBase,
    name: "Add",
    evaluator: "@{out:result}=@{in:a}+@{in:b}"
});
namespace.registerComponent("sub", {
    ...arithmeticBase,
    name: "Subtract",
    evaluator: "@{out:result}=@{in:a}-@{in:b}"
});
namespace.registerComponent("div", {
    ...arithmeticBase,
    name: "Divide",
    evaluator: "@{out:result}=@{in:a}/@{in:b}"
});
namespace.registerComponent("mult", {
    ...arithmeticBase,
    name: "Multiply",
    evaluator: "@{out:result}=@{in:a}*@{in:b}"
});
namespace.registerComponent("exp", {
    ...arithmeticBase,
    name: "Exponentiation",
    evaluator: "@{out:result}=@{in:a}**@{in:b}"
});
namespace.registerComponent("mod", {
    ...arithmeticBase,
    name: "Modulus",
    evaluator: "@{out:result}=@{in:a}%@{in:b}"
});
namespace.registerComponent("inc", {
    name: "Increase",
    inputs: {
        a: {
            name: "A",
            type: "number"
        },
    },
    outputs: {
        result: {
            name: "Result",
            type: "number"
        }
    },
    evaluator: "@{out:result}=@{in:a}++"
});
namespace.registerComponent("dec", {
    name: "Decrease",
    inputs: {
        a: {
            name: "A",
            type: "number"
        },
    },
    outputs: {
        result: {
            name: "Result",
            type: "number"
        }
    },
    evaluator: "@{out:result}=@{in:a}--"
});
//# sourceMappingURL=core_math.js.map