import { registerNamespace } from "./../component_manager.js";
////////////////////////////////////////////////
// Core.Memory.Array
////////////////////////////////////////////////
let ns_core_memory_array = registerNamespace("core.memory.array", {
    name: "Array",
    defaultIcon: "data_array",
    description: ``,
});
ns_core_memory_array.registerComponent("array", {
    name: "Array",
    outputs: {
        "array": {
            name: "Array",
            type: "array"
        }
    },
    evaluator: `@{out:array}=[];`
});
ns_core_memory_array.registerComponent("array_get", {
    name: "Get",
    inputs: {
        "array": {
            name: "Array",
            type: "array"
        },
        "index": {
            name: "Index",
            type: "number"
        }
    },
    outputs: {
        "value": {
            name: "Value",
            type: "any"
        }
    },
    evaluator: `@{out:value}=@{in:array}[@{in:index}];`
});
ns_core_memory_array.registerComponent("array_set", {
    name: "Set",
    inputs: {
        "array": {
            name: "Array",
            type: "array"
        },
        "index": {
            name: "Index",
            type: "number"
        },
        "value": {
            name: "Value",
            type: "any"
        }
    },
    outputs: {
        "value": {
            name: "Value",
            type: "any"
        }
    },
    evaluator: `
        @{in:array}[@{in:index}]=@{in:value};
        @{out:array}=@{in:array};`
});
ns_core_memory_array.registerComponent("array_push", {
    name: "Push",
    inputs: {
        "array": {
            name: "Array",
            type: "array"
        },
        "value": {
            name: "Value",
            type: "any",
            isMulti: true
        }
    },
    outputs: {
        "array": {
            name: "Array",
            type: "array"
        }
    },
    evaluator: () => {
        let str = "";
        return `
            @{out:array}=@{in:array};
            @{in:array}.push.apply(@{in:array}, [${str}]);
            `;
    }
});
ns_core_memory_array.registerComponent("array_length", {
    name: "Length",
    inputs: {
        "array": {
            name: "Array",
            type: "array"
        },
    },
    outputs: {
        "length": {
            name: "Length",
            type: "number"
        }
    },
    evaluator: `
        @{out:length}=@{in:array}.length;`
});
ns_core_memory_array.registerComponent("each", {
    name: "Each",
    inputs: {
        "array": {
            name: "Array",
            type: "array"
        },
    },
    outputs: {
        "value": {
            name: "Value",
            type: "any",
            signal: true,
        },
        "finished": {
            name: "Finished",
            type: "number",
            signal: true,
        },
        "index": {
            name: "Index",
            type: "number"
        },
    },
    evaluator: `
        var len = @{in:array}.length;
        $for(0, len, 1,
            function (i) {
                @{out:index}=i;
                @{out:value}=@{in:array}[i];
                @{triggerSignal:value};
            },
            function () {
                @{out:finished}=len;
                @{triggerSignal:finished};
            });
            `
});
//# sourceMappingURL=core_memory_array.js.map