import Connection from "./connection.js";
import Group from "./group.js";
import Node, { PortMap, KeyValueMap } from "./node.js";
import Patch, { GroupMap, NodeMap } from "./patch.js";
import Port from "./port.js";
import Project from "./project.js";
export declare type ConnectionMapSchema = {
    [key: string]: (InputConnectionSchema | OutputConnectionSchema)[];
};
export declare type PropertyMapSchema = {
    [key: string]: PropertySchema;
};
export declare type NodeMapSchema = {
    [key: number]: NodeSchema;
};
export declare type ConnectionSchema = InputConnectionSchema & OutputConnectionSchema;
export interface InputConnectionSchema {
    id: number;
    fromNodeId: number;
    fromPortKey: string;
}
export interface OutputConnectionSchema {
    id: number;
    toNodeId: number;
    toPortKey: string;
}
export interface PropertySchema {
    value: any;
}
export interface PropertySchemaMap {
    [key: string]: any;
}
export interface NodeSchema {
    type?: number;
    name?: string;
    id: number;
    componentKey: string;
    inputs?: ConnectionMapSchema;
    outputs?: ConnectionMapSchema;
    properties?: PropertySchemaMap;
    metadata?: PropertySchemaMap;
    patch?: PatchSchema;
}
export interface PatchSchema {
    nodes: NodeMapSchema;
    groups?: GroupSchema[];
}
export interface ProjectSchema {
    name: string;
    patch: PatchSchema;
}
export interface GroupSchema {
    name: string;
    comment: string;
    nodeIdList: number[];
}
export interface ExportOptions {
    excludeInputs?: boolean;
    excludeOutputs?: boolean;
}
export declare const exportProject: (project: Project, options?: ExportOptions | undefined) => ProjectSchema;
export declare const exportPatch: (patch: Patch, options?: ExportOptions | undefined) => PatchSchema;
export declare const exportGroup: (group: Group) => {
    name: string;
    comment: string;
    nodeIdList: number[];
};
export declare const exportGroups: (groupsMap: GroupMap) => GroupSchema[];
export declare const exportNodes: (nodes: NodeMap, options?: ExportOptions | undefined) => NodeMapSchema;
export declare const exportNode: (node: Node, options?: ExportOptions | undefined) => NodeSchema;
export declare const exportConnections: (ports: PortMap, connectionExportFn: (connection: Connection) => any) => ConnectionMapSchema;
export declare const exportPortConnections: (port: Port, connectionExportFn: (connection: Connection) => any) => (InputConnectionSchema | OutputConnectionSchema)[];
export declare const exportInputConnection: (connection: Connection) => {
    id: number;
    fromNodeId: number;
    fromPortKey: string;
};
export declare const exportOutputConnection: (connection: Connection) => {
    id: number;
    toNodeId: number;
    toPortKey: string;
};
export declare const exportConnection: (connection: Connection) => ConnectionSchema;
export declare const exportKeyValues: (properties: KeyValueMap) => PropertyMapSchema;
