import { NodeType } from "./node.js";
export const exportProject = (project, options) => {
    return {
        name: project.name,
        patch: exportPatch(project.mainPatch, { ...options, ...{ excludeOutputs: true } })
    };
};
export const exportPatch = (patch, options) => {
    let patchSchema = {
        nodes: exportNodes(patch.nodes, options),
    };
    if (patch.groups) {
        patchSchema.groups = exportGroups(patch.groups);
    }
    return patchSchema;
};
export const exportGroup = (group) => {
    return {
        name: group.name,
        comment: group.comment,
        nodeIdList: [...group.nodeIdList]
    };
};
export const exportGroups = (groupsMap) => {
    let groupSchemas = [];
    for (let group of Object.values(groupsMap)) {
        groupSchemas.push(exportGroup(group));
    }
    return groupSchemas;
};
export const exportNodes = (nodes, options) => {
    let result = {};
    for (let [id, node] of Object.entries(nodes)) {
        result[+id] = exportNode(node, options);
    }
    return result;
};
export const exportNode = (node, options) => {
    let nodeSchema = {
        id: node.id,
        componentKey: node.componentIdentKey,
    };
    if (node.type != NodeType.Normal) {
        nodeSchema.type = node.type;
    }
    if (node.name && node.name != "") {
        nodeSchema.name = node.name;
    }
    if (node.type == NodeType.Module) {
        nodeSchema.patch = exportPatch(node.patch, options);
    }
    if (node.properties != null && Object.keys(node.properties).length > 0) {
        nodeSchema.properties = exportKeyValues(node.properties);
    }
    if (node.metadata != null && Object.keys(node.metadata).length > 0) {
        nodeSchema.metadata = exportKeyValues(node.metadata);
    }
    if ((options === null || options === void 0 ? void 0 : options.excludeInputs) !== true && node.inputs != null && Object.keys(node.inputs).length > 0) {
        nodeSchema.inputs = exportConnections(node.inputs, exportInputConnection);
    }
    if ((options === null || options === void 0 ? void 0 : options.excludeOutputs) !== true && node.outputs != null && Object.keys(node.outputs).length > 0) {
        nodeSchema.outputs = exportConnections(node.outputs, exportOutputConnection);
    }
    return nodeSchema;
};
export const exportConnections = (ports, connectionExportFn) => {
    let connectionMap = {};
    for (let [portKey, port] of Object.entries(ports)) {
        connectionMap[portKey] = exportPortConnections(port, connectionExportFn);
    }
    return connectionMap;
};
export const exportPortConnections = (port, connectionExportFn) => {
    let connections = [];
    for (let [_, connection] of port.connections) {
        connections.push(connectionExportFn(connection));
    }
    return connections;
};
export const exportInputConnection = (connection) => {
    return {
        id: connection.id,
        fromNodeId: connection.outputPort.ownerNode.id,
        fromPortKey: connection.outputPort.key
    };
};
export const exportOutputConnection = (connection) => {
    return {
        id: connection.id,
        toNodeId: connection.inputPort.ownerNode.id,
        toPortKey: connection.inputPort.key
    };
};
export const exportConnection = (connection) => {
    let inputPort = connection.inputPort;
    let outputPort = connection.outputPort;
    return {
        id: connection.id,
        fromNodeId: outputPort.ownerNode.id,
        fromPortKey: outputPort.key,
        toNodeId: inputPort.ownerNode.id,
        toPortKey: inputPort.key
    };
};
export const exportKeyValues = (properties) => {
    let result = {};
    for (let [key, value] of Object.entries(properties)) {
        result[key] = value;
    }
    return result;
};
//# sourceMappingURL=export.js.map