import Group from "./group.js";
import { Normalizer } from "./id_generator.js";
import { NodeType } from "./node.js";
import Patch from "./patch.js";
import Project from "./project.js";
const noNormalize = {
    get: (id) => { return id; }
};
export const importProject = (projectSchema, normalize = false) => {
    let project = new Project(projectSchema.name);
    let normalizer = normalize ? new Normalizer(project.idGenerator) : undefined;
    project.mainPatch = project.activePatch = importPatch(projectSchema.patch, normalizer);
    return project;
};
export const importPatch = (patchSchema, normalizer = noNormalize) => {
    let patch = new Patch(null);
    patch.groups = patchSchema.groups ? importGroups(patchSchema.groups) : {};
    // Nodes
    for (let [_, nodeSchema] of Object.entries(patchSchema.nodes)) {
        importNode(nodeSchema, patch, normalizer);
    }
    // Connections
    for (let [_, nodeSchema] of Object.entries(patchSchema.nodes)) {
        importConnections(nodeSchema, patch, normalizer);
    }
    return patch;
};
export const importGroups = (groupSchemas) => {
    let groupMap = {};
    let idCounter = 0;
    for (let groupSchema of Object.values(groupSchemas)) {
        let group = new Group(idCounter++, groupSchema.name, groupSchema.comment, groupSchema.nodeIdList);
        groupMap[group.id] = group;
    }
    return groupMap;
};
const createKeyValueMap = (keyValueMap) => {
    let map = {};
    if (keyValueMap) {
        for (let [key, value] of Object.entries(keyValueMap)) {
            map[key] = value;
        }
        return map;
    }
    return undefined;
};
export const importNode = (nodeSchema, patch, normalizer = noNormalize) => {
    let node = patch.createNode(normalizer.get(nodeSchema.id), nodeSchema.componentKey, createKeyValueMap(nodeSchema.properties), createKeyValueMap(nodeSchema.metadata));
    // Module
    if (nodeSchema.type == NodeType.Module && nodeSchema.patch != null) {
        node.patch = importPatch(nodeSchema.patch, normalizer);
        node.patch.ownerModuleNode = node;
    }
    return node;
};
export const importConnections = (nodeSchema, patch, normalizer = noNormalize) => {
    let targetNode = patch.getNode(normalizer.get(nodeSchema.id));
    if (nodeSchema.inputs != null) {
        for (let [portKey, ports] of Object.entries(nodeSchema.inputs)) {
            for (let conn of ports) {
                let fromNode = patch.getNode(normalizer.get(conn.fromNodeId));
                fromNode.createConnection(normalizer.get(conn.id), conn.fromPortKey, targetNode, portKey);
            }
        }
    }
    else if (nodeSchema.outputs != null) {
        for (let [portKey, ports] of Object.entries(nodeSchema.outputs)) {
            for (let conn of ports) {
                let toNode = patch.getNode(normalizer.get(conn.toNodeId));
                targetNode.createConnection(normalizer.get(conn.id), portKey, toNode, conn.toPortKey);
            }
        }
    }
};
//# sourceMappingURL=import.js.map