import Port, { PortType } from "./port.js";
import Connection from "./connection.js";
import { Input, Port as ComponentPort } from "./component_manager.js";
import Component from "./component.js";
import Patch from "./patch.js";
export declare enum NodeType {
    Normal = 0,
    Module = 1,
    Group = 2,
    ModuleInput = 3,
    ModuleOutput = 4,
    Internal = 5
}
export declare type PortMap = {
    [portKey: string]: Port;
};
export declare type KeyValueMap = {
    [key: string]: any;
};
export declare type MetadataMap = {
    [key: string]: any;
};
export declare type ConnectionEventData = {
    connection: Connection;
    inputIndex: number | undefined;
};
export declare type PropertyEventData = {
    node: Node;
    key: string;
    value: any;
};
export declare type MetadataEventData = {
    node: Node;
    key: string;
    value: any;
};
export declare type SwapInputConnectionsEventData = {
    port: Port;
    index1: number;
    index2: number;
};
declare class Node {
    type: NodeType;
    name: string;
    id: number;
    componentIdentKey: string;
    component: Component;
    outputs: PortMap;
    inputs: PortMap;
    properties: KeyValueMap;
    metadata: MetadataMap;
    ownerPatch: Patch;
    constructor(ownerPatch: Patch, id: number, componentIdentKey: string);
    isConnectableInputPort(key: string): boolean;
    isConnectableOutputPort(key: string): boolean;
    createFlowPort(): ComponentPort;
    getConnectableInputPorts(): [string, Input][] | string[][];
    getConnectableOutputPorts(): [string, Input][] | string[][];
    lazyGetPort(type: PortType, portKey: string): Port;
    getOutputPort(portKey: string): Port;
    getInputPort(portKey: string): Port;
    findConnection(connectionId: number): Connection | null;
    removeOldInputConnection(targetNode: Node, targetPortKey: string): void;
    createConnection(id: number, fromPortKey: string, targetNode: Node, targetPortKey: string, inputIndex?: number): Connection;
    removeOutputConnection(portKey: string, connectionId: number | null): Connection[];
    removeInputConnection(portKey: string, connectionId: number | null): void;
    swapInputConnections(portKey: string, index1: number, index2: number): void;
    isPropertySet(key: string): boolean;
    setProperty(key: string, value: any): void;
    getProperty(key: string): any;
    removeProperty(key: string): void;
    isMetadataSet(key: string): boolean;
    setMetadata(key: string, value: any): void;
    getMetadata(key: string): any;
    removeMetadata(key: string): void;
    getName(): string;
}
export default Node;
