import { ORIGIN_PATCH_KEY, ORIGIN_PROJECT_KEY } from "./actions.js";
import { HistoryManager } from "./history_manager.js";
import IDGenerator from "./id_generator.js";
import observer from "./observer.js";
import Patch from "./patch.js";
import { Events } from "./events.js";
class Project {
    constructor(name) {
        this.name = "";
        this.historyManager = new HistoryManager();
        this.idGenerator = new IDGenerator();
        this.name = name;
        this.activePatch = this.mainPatch = new Patch(null);
        // Store active patch on every action/history data.
        this.historyManager.addEventListener("onBeforeDo", (data) => {
            data.set(ORIGIN_PATCH_KEY, this.getActivePatch());
        });
        this.historyManager.addEventListener("onCreateActionData", (data) => {
            data.set(ORIGIN_PROJECT_KEY, this);
        });
    }
    getActivePatch() {
        return this.activePatch;
    }
    setActivePatch(patch) {
        this.activePatch = patch;
        observer.triggerEvent(Events.OnSetActivePatch, patch);
        return patch;
    }
    reset() {
        observer.triggerEvent(Events.OnReset);
        this.idGenerator.reset();
        this.historyManager.reset();
        this.activePatch = this.mainPatch = new Patch(null);
    }
}
export default Project;
//# sourceMappingURL=project.js.map