import { ComponentData, Namespace } from "./component_manager.js";

export const FLOW_PORT_KEY = "_flow_port_";

class Component {
    key: string;
    data: ComponentData;
    namespace: Namespace;

    constructor(namespace: Namespace, key: string, data: ComponentData) {
        this.namespace = namespace;
        this.key = key;
        this.data = data;
    }

    getFullKey() {
        return this.namespace.key + "." + this.key;
    }

    hasProperty(key: string) {
        if (this.data.properties != null) {
            return key in this.data.properties;
        }
        return false;
    }

    hasInputPort(key: string) {
        return key == FLOW_PORT_KEY || (this.data.inputs != null && key in this.data.inputs);
    }

    hasOutputPort(key: string) {
        return key == FLOW_PORT_KEY || this.data.outputs != null && key in this.data.outputs;
    }

    getInputPort(key: string) {
        return this.data.inputs != null ? this.data.inputs[key] : null;
    }

    getOutputPort(key: string) {
        return this.data.outputs != null ? this.data.outputs[key] : null;
    }

    getName() {
        return this.namespace.data.name + "." + this.data.name;
    }

    getInputs() {
        let inputs = this.data.inputs;
        return inputs ? inputs : {};
    }

    getOutputs() {
        let outputs = this.data.outputs;
        return outputs ? outputs : {};
    }
}

export default Component;