import { registerNamespace } from "./../component_manager.js";

let namespace = registerNamespace("core.logic", {
    name: "Logic",
    defaultIcon: "mediation",
    description: ``
});

namespace.registerComponent("true", {
    name: "True",
    outputs: {
        "value": {
            name: "Value",
            type: "boolean"
        },
    },
    evaluator: `@{out:value}=true;`
});

namespace.registerComponent("false", {
    name: "False",
    outputs: {
        "value": {
            name: "Value",
            type: "boolean"
        },
    },
    evaluator: `@{out:value}=false;`
});

namespace.registerComponent("condition", {
    name: "Condition",
    inputs: {
        "cond": {
            name: "Condition",
            type: "boolean"
        },
    },
    outputs: {
        "true": {
            name: "True",
            type: "boolean",
            signal: true
        },
        "false": {
            name: "False",
            type: "boolean",
            signal: true
        }
    },
    evaluator: `if (@{in:cond}) @{triggerSignal:true}; else @{triggerSignal:false};`
});

namespace.registerComponent("gate", {
    name: "Gate",
    inputs: {
        "condition": {
            name: "Condition",
            type: "boolean"
        },
        "value": {
            name: "Value",
            type: "any"
        },
    },
    outputs: {
        "value": {
            name: "Value",
            type: "any",
            signal: true,
        },
    },
    evaluator: `if (@{in:condition}) {
        @{out:value}=@{in:value}
        @{triggerSignal:value};
    }`
});

namespace.registerComponent("and", {
    name: "And",
    inputs: {
        "a": {
            name: "A",
            type: "any"
        },
        "b": {
            name: "B",
            type: "any"
        },
    },
    outputs: {
        "result": {
            name: "Result",
            type: "boolean"
        },
    },
    evaluator: `@{out:result}=@{in:a}&&@{in:b};`
});

namespace.registerComponent("or", {
    name: "Or",
    inputs: {
        "a": {
            name: "A",
            type: "any"
        },
        "b": {
            name: "B",
            type: "any"
        },
    },
    outputs: {
        "result": {
            name: "Result",
            type: "boolean"
        },
    },
    evaluator: `@{out:result}=@{in:a}||@{in:b};`
});

namespace.registerComponent("compare", {
    name: "Compare",
    inputs: {
        "a": {
            name: "A",
            type: "any"
        },
        "b": {
            name: "B",
            type: "any"
        },
    },
    outputs: {
        "true": {
            name: "True",
            type: "boolean",
            signal: true
        },
        "false": {
            name: "False",
            type: "boolean",
            signal: true
        },
    },
    evaluator: `if (@{in:a} == @{in:b}) @{triggerSignal:true}; else @{triggerSignal:false};`
});

namespace.registerComponent("latch", {
    name: "Latch",
    inputs: {
        "flow": {
            name: "Flow",
            type: "any",
        },
        "value": {
            name: "Value",
            type: "any"
        }
    },
    outputs: {
        "value": {
            name: "value",
            type: "any"
        }
    },
    evaluator: `@{out:value}=@{in:value}`
});
