//type CallbackFn = <T> (arg: T | null) => void;

type ListenersMap = {
    [eventName: string]: Function[]
};

class EventHandler {
    listenersMap: ListenersMap = {};
    addEventListener(eventName: string, callback: Function) {
        if (this.listenersMap[eventName] == null) {
            this.listenersMap[eventName] = [];
        }
        let idx = this.listenersMap[eventName].length;
        this.listenersMap[eventName].push(callback);
        return idx;
    }

    triggerEvent<T>(eventName: string, arg: T | null = null) {
        let listeners = this.listenersMap[eventName];

        if (listeners == null) {
            return;
        }

        for (let listener of listeners) {
            listener(<T>arg);
        }
    }
}

export default EventHandler;
