import { Events } from "./events";
import observer from "./observer";

export interface GroupEventData {
    groupId: number;
    nodeId: number;
}

class Group {
    id: number;
    name: string = "New Group";
    comment: string = "";
    nodeIdList: number[];

    constructor(id: number, name: string, comment = "", nodeIdList: number[] = []) {
        this.id = id;
        this.name = name;
        this.comment = comment;
        this.nodeIdList = nodeIdList;
    }

    addNodeId(id: number) {
        if (this.nodeIdList.indexOf(id) < 0) {
            this.nodeIdList.push(id);
            observer.triggerEvent<GroupEventData>(Events.OnAddNodeToGroup, {
                groupId: this.id,
                nodeId: id
            });
        }
    }

    removeNodeId(id: number) {
        let idx = this.nodeIdList.indexOf(id);
        if (idx >= 0) {
            this.nodeIdList.splice(idx, 1);
            observer.triggerEvent<GroupEventData>(Events.OnRemoveNodeFromGroup, {
                groupId: this.id,
                nodeId: id,
            });
        }
    }

    hasNodeId(id: number) {
        return this.nodeIdList.indexOf(id) >= 0;
    }
}

export default Group;