import Node from "./node.js";
import { ConnectionSchema, exportConnection } from "./export.js";

export type RelatedConnections = {
    [nodeId: number]: {
        inputs: ConnectionSchema[];
        outputs: ConnectionSchema[];
    }
}

export const getRelatedConnections = (nodes: Node[], mode: ("external" | "internal")) => {
    const relatedMap: RelatedConnections = {};
    const lazyCreateEntry = (id: number) => {
        if (!(id in relatedMap)) {
            relatedMap[id] = {
                outputs: [],
                inputs: []
            }
        }
    }
    const idExist = (id: number) => {
        //let c = nodes.filter(n => n.id == id);
        let c = nodes.every(n => n.id != id);
        return (mode == "external" ? c : !c);
    }
    for (let node of Object.values(nodes)) {
        for (let port of Object.values(node.inputs)) {
            for (let [_, connection] of port.connections) {
                if (idExist(connection.outputPort.ownerNode.id)) {
                    lazyCreateEntry(node.id);
                    relatedMap[node.id].inputs.push(exportConnection(connection));
                }
            }
        }
        for (let port of Object.values(node.outputs)) {
            for (let [_, connection] of port.connections) {
                if (idExist(connection.inputPort.ownerNode.id)) {
                    lazyCreateEntry(node.id);
                    relatedMap[node.id].outputs.push(exportConnection(connection));
                }
            }
        }
    }
    return relatedMap;
}